#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

# import standard library

# import interfaces
from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.features.renderer import IRenderedContent

# import packages
from pyams_file.schema import ImageField
from pyams_gis.schema import GeoPointField
from pyams_i18n.schema import I18nTextLineField
from zope.schema import TextLine, Text, Choice

from pyams_content import _


#
# Contact paragraph
#

CONTACT_PARAGRAPH_TYPE = 'Contact'
CONTACT_PARAGRAPH_RENDERERS = 'PyAMS.paragraph.contact.renderers'


class IContactParagraph(IRenderedContent, IBaseParagraph):
    """Contact paragraph interface"""

    name = TextLine(title=_("Contact identity"),
                    description=_("Name of the contact"),
                    required=True)

    charge = I18nTextLineField(title=_("In charge of"),
                               description=_("Label of contact function"),
                               required=False)

    photo = ImageField(title=_("Photo"),
                       description=_("Use 'browse' button to select contact picture"),
                       required=False)

    gps_location = GeoPointField(title=_("GPS location"),
                                 description=_("GPS coordinates used to locate contact"),
                                 required=False)

    address = Text(title=_("Address"),
                   required=False)

    renderer = Choice(title=_("Contact template"),
                      description=_("Presentation template used for this contact"),
                      vocabulary=CONTACT_PARAGRAPH_RENDERERS,
                      default='default')
