#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IBaseParagraph
from pyams_content.features.renderer.interfaces import IRenderedContent

# import packages
from pyams_file.schema import VideoField
from pyams_i18n.schema import I18nHTMLField, I18nTextField
from zope.schema import TextLine, Choice

from pyams_content import _


#
# HTML paragraph
#

VIDEO_PARAGRAPH_TYPE = 'Video'
VIDEO_PARAGRAPH_RENDERERS = 'PyAMS.paragraph.video.renderers'


class IVideoParagraph(IRenderedContent, IBaseParagraph):
    """Video paragraph"""

    body = I18nHTMLField(title=_("Body"),
                         required=False)

    description = I18nTextField(title=_("Description"),
                                description=_("File description displayed by front-office template"),
                                required=False)

    author = TextLine(title=_("Author"),
                      description=_("Name of document's author"),
                      required=False)

    data = VideoField(title=_("Video data"),
                      description=_("Video file content"),
                      required=True)

    renderer = Choice(title=_("Video template"),
                      description=_("Presentation template used for this video"),
                      vocabulary=VIDEO_PARAGRAPH_RENDERERS,
                      default='default')
