#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IBaseParagraph, IParagraphRenderer
from pyams_content.component.paragraph.interfaces.contact import CONTACT_PARAGRAPH_TYPE, IContactParagraph
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerView, IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from transaction.interfaces import ITransactionManager
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.paragraph.contact import ContactParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAddMenu, BaseParagraphAJAXAddForm, \
    BaseParagraphPropertiesEditForm, BaseParagraphAJAXEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='add-contact-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=540)
class ContactParagraphAddMenu(BaseParagraphAddMenu):
    """Contact paragraph add menu"""

    label = _("Contact card...")
    label_css_class = 'fa fa-fw fa-id-card-o'
    url = 'add-contact-paragraph.html'
    paragraph_type = CONTACT_PARAGRAPH_TYPE


@pagelet_config(name='add-contact-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ContactParagraphAddForm(AdminDialogAddForm):
    """Contact paragraph add form"""

    legend = _("Add new contact card")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-id-card-o'

    fields = field.Fields(IContactParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-contact-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ContactParagraphAddForm, self).updateWidgets(prefix)
        if 'address' in self.widgets:
            self.widgets['address'].widget_css_class = 'textarea'

    def create(self, data):
        return ContactParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-contact-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContactParagraphAJAXAddForm(BaseParagraphAJAXAddForm, ContactParagraphAddForm):
    """Contact paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IContactParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ContactParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Contact paragraph properties edit form"""

    legend = _("Edit contact card properties")
    icon_css_class = 'fa fa-fw fa-id-card-o'

    fields = field.Fields(IContactParagraph).omit('__parent__', '__name__', 'visible')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ContactParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'address' in self.widgets:
            self.widgets['address'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IContactParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContactParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, ContactParagraphPropertiesEditForm):
    """Contact paragraph properties edit form, JSON renderer"""


@adapter_config(context=(IContactParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class ContactParagraphInnerEditForm(ContactParagraphPropertiesEditForm):
    """Contact paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IContactParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContactParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, ContactParagraphInnerEditForm):
    """Contact paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(ContactParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IBaseParagraph, ())
        if 'title' in updated:
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                    'object_name': self.context.__name__,
                    'title': II18n(self.context).query_attribute('title', request=self.request),
                    'visible': self.context.visible
                }
            })
        updated = changes.get(IContactParagraph, ())
        if ('photo' in updated) or ('renderer' in updated):
            # we have to commit transaction to be able to handle blobs...
            ITransactionManager(self.context).get().commit()
            form = ContactParagraphInnerEditForm(self.context, self.request)
            form.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'object_id': '{0}_{1}_{2}'.format(
                        self.context.__class__.__name__,
                        getattr(form.getContent(), '__name__', 'noname').replace('++', ''),
                        form.id),
                    'content': form.render()
                }
            })
        return output


#
# Contact paragraph renderer
#

@adapter_config(context=(IContactParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class ContactParagraphRenderer(BaseRenderedContentRenderer):
    """Contact paragraph renderer"""
