#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphRenderer
from pyams_content.component.paragraph.interfaces.header import IHeaderParagraph, HEADER_PARAGRAPH_TYPE
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.paragraph.header import HeaderParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='add-header-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=40)
class HeaderParagraphAddMenu(BaseParagraphAddMenu):
    """Header paragraph add menu"""

    label = _("Header...")
    label_css_class = 'fa fa-fw fa-header'
    url = 'add-header-paragraph.html'
    paragraph_type = HEADER_PARAGRAPH_TYPE


@pagelet_config(name='add-header-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HeaderParagraphAddForm(AdminDialogAddForm):
    """Header paragraph add form"""

    legend = _("Add new header paragraph")
    icon_css_class = 'fa fa-fw fa-header'

    fields = field.Fields(IHeaderParagraph).select('header', 'renderer')
    ajax_handler = 'add-header-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HeaderParagraphAddForm, self).updateWidgets(prefix)
        if 'header' in self.widgets:
            self.widgets['header'].widget_css_class = 'textarea height-100'

    def create(self, data):
        return HeaderParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-header-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HeaderParagraphAJAXAddForm(BaseParagraphAJAXAddForm, HeaderParagraphAddForm):
    """Header paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IHeaderParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HeaderParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Header paragraph properties edit form"""

    legend = _("Edit header paragraph properties")
    icon_css_class = 'fa fa-fw fa-header'

    fields = field.Fields(IHeaderParagraph).select('header', 'renderer')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HeaderParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'header' in self.widgets:
            self.widgets['header'].widget_css_class = 'textarea height-100'


@view_config(name='properties.json', context=IHeaderParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HeaderParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, HeaderParagraphPropertiesEditForm):
    """Header paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(HeaderParagraphPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'header' in changes.get(IHeaderParagraph, ()):
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                    'object_name': self.context.__name__,
                    'title': II18n(self.context).query_attribute('title', request=self.request),
                    'visible': self.context.visible
                }
            })
        return output


@adapter_config(context=(IHeaderParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class HeaderParagraphInnerEditForm(HeaderParagraphPropertiesEditForm):
    """Header paragraph inner edit form"""

    legend = None
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IHeaderParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HeaderParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, HeaderParagraphInnerEditForm):
    """Header paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(HeaderParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IHeaderParagraph, ())
        if 'renderer' in updated:
            form = HeaderParagraphInnerEditForm(self.context, self.request)
            form.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshWidget',
                    'parent_id': '{0}_{1}_{2}'.format(
                        self.context.__class__.__name__,
                        getattr(form.getContent(), '__name__', 'noname').replace('++', ''),
                        form.id),
                    'widget_name': form.widgets['renderer'].name,
                    'content': form.widgets['renderer'].render()
                }
            })
        return output


#
# Header paragraph renderer
#

@adapter_config(context=(IHeaderParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class HeaderParagraphRenderer(BaseRenderedContentRenderer):
    """Header paragraph renderer"""
