#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IBaseParagraph
from zope.annotation import IAttributeAnnotatable
from zope.contentprovider.interfaces import IContentProvider

# import packages
from pyams_i18n.schema import I18nHTMLField, I18nTextField
from zope.interface import Interface, Attribute
from zope.schema import Choice, TextLine

from pyams_content import _


class IExternalVideoSettings(Interface):
    """External video settings"""


class IExternalVideoProvider(Interface):
    """External video provider"""

    settings_interface = Attribute("Video provider settings interface")


class IExternalVideo(IAttributeAnnotatable):
    """Base interface for external video integration"""

    description = I18nTextField(title=_("Description"),
                                description=_("File description displayed by front-office template"),
                                required=False)

    author = TextLine(title=_("Author"),
                      description=_("Name of document's author"),
                      required=False)

    provider_name = Choice(title=_("Video provider"),
                           description=_("Name of external platform providing selected video"),
                           required=False,
                           vocabulary="PyAMS video providers")

    def get_provider(self):
        """Get external video provider utility"""

    settings = Attribute("Video settings")


EXTERNAL_VIDEO_PARAGRAPH_TYPE = 'External video'


class IExternalVideoParagraph(IExternalVideo, IBaseParagraph):
    """External video paragraph"""

    body = I18nHTMLField(title=_("Body"),
                         required=False)


class IExternalVideoRenderer(IContentProvider):
    """External video renderer"""
