#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_content.shared.form.interfaces import IFormFieldContainerTarget, IFormFieldContainer
from pyams_form.interfaces.form import IFormHelp
from pyams_i18n.interfaces import II18n
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_content.features.renderer.zmi import BaseContentRenderer
from pyams_form.form import InnerAddForm
from pyams_form.help import FormHelp
from pyams_utils.adapter import adapter_config
from z3c.form import field, button
from zope.interface import Interface


class FormFieldContainerDisplayForm(InnerAddForm):
    """Form fields container display form"""

    @property
    def legend(self):
        return II18n(self.context).query_attribute('user_title', request=self.request)

    @property
    def fields(self):
        fields = field.Fields(*IFormFieldContainer(self.context).get_fields())
        if self.context.use_captcha:
            # TODO: add captcha
            # fields += field.Fields(Captcha(title='', description='', required=True))
            pass
        return fields

    buttons = button.Buttons(Interface)

    def updateWidgets(self, prefix=None):
        super(FormFieldContainerDisplayForm, self).updateWidgets(prefix)
        for widget in self.widgets.values():
            field = IFormFieldContainer(self.context).get(widget.field.__name__)
            if field is not None:
                widget.placeholder = field.placeholder
                if field.field_type == 'text':
                    widget.widget_css_class = 'textarea'


@adapter_config(context=(IFormFieldContainerTarget, IPyAMSLayer, FormFieldContainerDisplayForm),
                provides=IFormHelp)
class FormFieldContainerDisplayFormHelp(FormHelp):
    """Form field container display form help adapter"""

    def __new__(cls, context, request, view):
        if not context.header:
            return None
        return FormHelp.__new__(cls)

    @property
    def message(self):
        return II18n(self.context).query_attribute('header', request=self.request)

    message_format = 'text'


@adapter_config(name='form-render', context=(IFormFieldContainerTarget, IPyAMSLayer),
                provides=IContentRenderer)
class FormFieldContainerRenderer(BaseContentRenderer):
    """Form field container renderer"""

    weight = 20
    display_form = None

    def __init__(self, context, request):
        super(FormFieldContainerRenderer, self).__init__(context, request)
        self.display_form = FormFieldContainerDisplayForm(context, self.request)

    def update(self):
        super(FormFieldContainerRenderer, self).update()
        self.display_form.update()

    def render(self):
        return self.display_form.render()
