#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphRenderer
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common import IWfSharedContent
from pyams_content.shared.logo.interfaces import ILogosParagraph, LOGOS_PARAGRAPH_TYPE
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.paragraph.zmi import IParagraphContainerView, BaseParagraphAddMenu, \
    BaseParagraphAJAXAddForm, BaseParagraphPropertiesEditForm, BaseParagraphAJAXEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_content.shared.logo.paragraph import LogosParagraph
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config, BaseContentProvider
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='add-logos-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=530)
class LogosParagraphAddMenu(BaseParagraphAddMenu):
    """Logos paragraph add menu"""

    label = _("Logos...")
    label_css_class = 'fa fa-fw fa-th-large'
    url = 'add-logos-paragraph.html'
    paragraph_type = LOGOS_PARAGRAPH_TYPE


@pagelet_config(name='add-logos-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class LogosParagraphAddForm(AdminDialogAddForm):
    """Logos paragraph add form"""

    legend = _("Add new logos paragraph")
    icon_css_class = 'fa fa-fw fa-th-large'

    fields = field.Fields(ILogosParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-logos-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def create(self, data):
        return LogosParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-logos-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class LogosParagraphAJAXAddForm(BaseParagraphAJAXAddForm, LogosParagraphAddForm):
    """Logos paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=ILogosParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class LogosParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Logos paragraph properties edit form"""

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit paragraph properties")
    icon_css_class = 'fa fa-fw fa-th-large'

    fields = field.Fields(ILogosParagraph).omit('__parent__', '__name__', 'visible')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION


@view_config(name='properties.json', context=ILogosParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class LogosParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, LogosParagraphPropertiesEditForm):
    """Logos paragraph properteis edit form, JSOn renderer"""


@adapter_config(context=(ILogosParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class LogosParagraphInnerEditForm(LogosParagraphPropertiesEditForm):
    """Logos paragraph properties inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=ILogosParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class LogosParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, LogosParagraphInnerEditForm):
    """Logos paragraph properteis inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(LogosParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(ILogosParagraph, ())
        if 'renderer' in updated:
            form = LogosParagraphInnerEditForm(self.context, self.request)
            form.update()
            content = form.getContent()
            output.setdefault('events', []).append({
                'evennt': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshWidget',
                    'parent_id': '{0}_{1}_{2}'.format(
                        content.__class__.__name__,
                        getattr(content, '__name__', 'noname').replace('++', ''),
                        form.id),
                    'widget_name': form.widgets['renderer'].name,
                    'content': form.widgets['renderer'].render()
                }
            })
        return output


#
# Logos paragraph renderer
#

@adapter_config(context=(ILogosParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class LogosParagraphRenderer(BaseRenderedContentRenderer):
    """Logos paragraph renderer"""
