#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.interfaces import IBaseContent
from pyams_sequence.interfaces import ISequentialIdTarget, IInternalReference
from pyams_workflow.interfaces import IWorkflowPublicationSupport
from zope.container.interfaces import IContainer, IContained

# import packages
from pyams_content.shared.common.interfaces import ISharedSite, IWfSharedContent, ISharedContent, \
    IBaseContentManagerRoles, IBaseSharedTool, IDeletableElement
from pyams_i18n.schema import I18nTextLineField
from zope.container.constraints import containers, contains
from zope.interface import Interface, Attribute
from zope.schema import Text, Bool

from pyams_content import _


class ISiteElement(IContained, IDeletableElement):
    """Base site element interface"""

    containers('.ISiteContainer')


class ISiteContainer(IContainer, IContained, IWorkflowPublicationSupport):
    """Base site container interface"""

    contains(ISiteElement)

    def get_folders_tree(self, selected=None):
        """Get site tree in JSON format"""


class ISiteFolder(IBaseContent, ISiteElement, ISiteContainer, ISequentialIdTarget):
    """Site folder interface

    A site folder is made to contain sub-folders and topics
    """

    notepad = Text(title=_("Notepad"),
                   description=_("Internal information to be known about this content"),
                   required=False)


class ISiteFolderFactory(Interface):
    """Site folder factory interface"""


class ISiteFolderRoles(IBaseContentManagerRoles):
    """Site folder roles interface"""


class ISiteManager(ISharedSite, ISiteContainer, IBaseSharedTool, IDeletableElement, ISequentialIdTarget):
    """Site manager interface"""

    contains(ISiteElement)

    folder_factory = Attribute("Folder factory")

    topic_content_type = Attribute("Topic content type")
    topic_content_factory = Attribute("Topic content factory")


class ISiteManagerFactory(Interface):
    """Site manager factory interface"""


TOPIC_CONTENT_TYPE = 'topic'
TOPIC_CONTENT_NAME = _("Topic")


class IWfTopic(IWfSharedContent):
    """Topic interface"""


class IWfTopicFactory(Interface):
    """Topic factory interface"""


class ITopic(ISharedContent, ISiteElement):
    """Workflow managed topic interface"""


class IContentLink(ISiteElement, IInternalReference):
    """Rented content interface"""

    alt_title = I18nTextLineField(title=_("Alternate title"),
                                  description=_("Content title, as shown in front-office"),
                                  required=False)

    visible = Bool(title=_("Visible?"),
                   description=_("If 'no', link is not visible"),
                   required=True,
                   default=True)

    def get_target(self):
        """Get reference target"""
